/*
  Copyright (C) 2009 Julien Jorge

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: julien.jorge@gamned.org
*/
#ifndef __MY_CONTEXT_HPP__
#define __MY_CONTEXT_HPP__

#include "my_exportable.hpp"
#include <sstream>
#include <map>

/* prdclaration de la classe des images pour pouvoir l'utiliser dans
   my_context. */
class image;

/*----------------------------------------------------------------------------*/
/** Un contexte spcifique  notre programme de traitement d'images. */
class my_context
{
public:
  void add( const std::string& n, image* img );
  void add( const std::string& n, my_exportable* e );

  my_exportable* get( const std::string& n ) const;
  image* get_image( const std::string& n ) const;

private:
  /** Les images manipules. */
  std::map<std::string, image*> m_images;

  /** D'autres objets intervenant dans le script (mailer, ...) */
  std::map<std::string, my_exportable*> m_export;

}; // class my_context

namespace script
{
  /** Spcialisation pour retourner une image  partir de son nom. On fait une
      prdclaration ici pour que le compilateur sache qu'elle existe. */
  template<>
  class string_to_arg<my_context, const image&>
  {
  public:
    static const image&
    convert( const my_context& context, const std::string& arg );
  }; // string_to_arg
}

#endif // __MY_CONTEXT_HPP__
