/*
  Copyright (C) 2009 Julien Jorge

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: julien.jorge@gamned.org
*/
#ifndef __SCRIPT_EXPLICIT_METHOD_CALLER_HPP__
#define __SCRIPT_EXPLICIT_METHOD_CALLER_HPP__

#include "method_caller.hpp"
#include <cstdlib>

namespace script
{
  /*--------------------------------------------------------------------------*/
  /** La classe de base pour les classes appelants les fonctions membre. Elle
      convertit la classe passe  method_caller::execute() vers une instance
      d'un type prdfinit. SelfClass est le type de la classe sur laquelle on
      appelle la mthode. Elle doit hriter de base_exportable. */
  template<typename SelfClass>
  class explicit_method_caller:
    public method_caller<typename SelfClass::script_context_type>
  {
  public:
    typedef typename SelfClass::script_context_type script_context_type;

  public:
    // explicit execute va effectivement appeler la mthode sur une instance de
    // SelfClass. Cette mthode sera dfinie dans method_caller_args_0 et
    // similaires pour grr la conversion des paramtres et leur nombre.
    virtual void explicit_execute
    ( SelfClass& self, const std::vector<std::string>& args,
      const script_context_type& context )  const = 0;

  private:
    virtual void execute
    ( base_exportable<script_context_type>* self,
      const std::vector<std::string>& args,
      const script_context_type& context ) const
    {
      SelfClass* s = dynamic_cast<SelfClass*>(self);

      if ( s!=NULL )
        explicit_execute(*s, args, context);
    }
  }; // explicit_method_caller
} // namespace script

#endif // __SCRIPT_EXPLICIT_METHOD_CALLER_HPP__
