/*
  Copyright (C) 2009 Julien Jorge

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: julien.jorge@gamned.org
*/
#ifndef __SCRIPT_METHOD_CALLER_HPP__
#define __SCRIPT_METHOD_CALLER_HPP__

#include <vector>
#include <string>

namespace script
{
  /*--------------------------------------------------------------------------*/
  /** Prdclaration de la classe de base pour celles dont on veut exporter les
      mthodes. */
  template<typename Context>
  class base_exportable;

  /*--------------------------------------------------------------------------*/
  /** La classe de base pour les classes appelants les fonctions membre. Elle
      est utilise pour stoker ces instances dans un seul conteneur. Pour
      dfinir une classe appelant une fonction membre, on prfrera utiliser
      explicit_method_caller. Context est le contexte d'excution, par exemple
      script_context. */
  template<typename Context>
  class method_caller
  {
  public:
    typedef Context script_context_type;

  public:
    virtual void execute
    ( base_exportable<script_context_type>* self,
      const std::vector<std::string>& args,
      const script_context_type& context ) const = 0;

  }; // class method_caller
} // namespace script

#endif // __SCRIPT_METHOD_CALLER_HPP__
