/*
  Copyright (C) 2009 Julien Jorge

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: julien.jorge@gamned.org
*/
#ifndef __SCRIPT_METHOD_CALLER_ARGS_HPP__
#define __SCRIPT_METHOD_CALLER_ARGS_HPP__

#include "script/explicit_method_caller.hpp"
#include "script/string_to_arg.hpp"

namespace script
{
  /*--------------------------------------------------------------------------*/
  /** Cette classe dfinit un driv de explicit_method_caller pour appeler une
      mthode donne d'une classe donne. Cette dernire est dfinit par
      SelfClass. L'adresse de la mthode est Member et elle ne doit prendre
      aucun paramtre et retourne une valeur de type R. */
  template< typename SelfClass, typename R,
            R (SelfClass::*Member)() >
  class method_caller_args_0
  {
  public:
    typedef R (SelfClass::*mem_fun_type)();
    typedef typename SelfClass::script_context_type script_context_type;

  public:
    class caller_type:
      public explicit_method_caller<SelfClass>
    {
    private:
      void explicit_execute
      ( SelfClass& self, const std::vector<std::string>& args,
        const script_context_type& context ) const
      {
        const mem_fun_type member(Member);
        (self.*member)();
      }
    }; // class caller_type;

  public:
    /** Une seule instance de caller_type est suffisante dans la mesure o il
      n'y a pas de variables membres (s_caller ne sert qu' faire la transition
      vers la mthode relle). On pourra ainsi utiliser son adresse et viter
      des allocations dynamiques. */
    static const caller_type s_caller;

  }; // class method_caller_args_0

  /*--------------------------------------------------------------------------*/
  /** Dfinition de method_caller_args_0::s_caller pour les paramtres de
      patron de classes donns. */
  template< typename SelfClass, typename R,
            R (SelfClass::*Member)() >
  const typename
  method_caller_args_0<SelfClass, R, Member>::caller_type
  method_caller_args_0<SelfClass, R, Member>::s_caller;

  /*--------------------------------------------------------------------------*/
  /** Cette classe dfinit un driv de explicit_method_caller pour appeler une
      mthode donne d'une classe donne. Cette dernire est dfinit par
      SelfClasse. L'adresse de la mthode est Member et doit prendre un unique
      paramtre dont le type est donn dans ArgsList. */
  template< typename SelfClass, typename R, typename A0,
            R (SelfClass::*Member)(A0) >
  class method_caller_args_1
  {
  public:
    typedef R (SelfClass::*mem_fun_type)(A0);
    typedef typename SelfClass::script_context_type script_context_type;

  public:
    class caller_type:
      public explicit_method_caller<SelfClass>
    {
    private:
      void explicit_execute
      ( SelfClass& self, const std::vector<std::string>& args,
        const script_context_type& context ) const
      {
        const mem_fun_type member(Member);
        (self.*member)
          ( string_to_arg<script_context_type, A0>::convert(context, args[0]) );
      }
    }; // class caller_type;

  public:
    /** Une seule instance de caller_type est suffisante dans la mesure o il
      n'y a pas de variables membres (s_caller ne sert qu' faire la transition
      vers la mthode relle). On pourra ainsi utiliser son adresse et viter
      des allocations dynamiques. */
    static const caller_type s_caller;

  }; // class method_caller_args_1

  /*--------------------------------------------------------------------------*/
  /** Dfinition de method_caller_args_1::s_caller pour les paramtres de
      patron de classes donns. */
  template< typename SelfClass, typename R, typename A0,
            R (SelfClass::*Member)(A0) >
  const typename
  method_caller_args_1<SelfClass, R, A0, Member>::caller_type
  method_caller_args_1<SelfClass, R, A0, Member>::s_caller;

  /*--------------------------------------------------------------------------*/
  /** Cette classe dfinit un driv de explicit_method_caller pour appeler une
      mthode donne d'une classe donne. Cette dernire est dfinit par
      SelfClasse. L'adresse de la mthode est Member et doit prendre deux
      paramtres dont les types sont donns par A0 et A1. */
  template< typename SelfClass, typename R, typename A0, typename A1,
            R (SelfClass::*Member)(A0, A1) >
  class method_caller_args_2
  {
  public:
    typedef R (SelfClass::*mem_fun_type)(A0, A1);
    typedef typename SelfClass::script_context_type script_context_type;

  public:
    class caller_type:
      public explicit_method_caller<SelfClass>
    {
    private:
      void explicit_execute
      ( SelfClass& self, const std::vector<std::string>& args,
        const script_context_type& context ) const
      {
        const mem_fun_type member(Member);
        (self.*member)
          ( string_to_arg<script_context_type, A0>::convert(context, args[0]),
            string_to_arg<script_context_type, A1>::convert(context, args[1]) );
      }
    }; // class caller_type;

  public:
    /** Une seule instance de caller_type est suffisante dans la mesure o il
        n'y a pas de variables membres (s_caller ne sert qu' faire la
        transition vers la mthode relle). On pourra ainsi utiliser son adresse
        et viter des allocations dynamiques. */
    static const caller_type s_caller;

  }; // class method_caller_args_2

  /*--------------------------------------------------------------------------*/
  /** Dfinition de method_caller_args_2::s_caller pour les paramtres de
      patron de classes donns. */
  template< typename SelfClass, typename R, typename A0, typename A1,
            R (SelfClass::*Member)(A0, A1) >
  const typename
  method_caller_args_2<SelfClass, R, A0, A1, Member>::caller_type
  method_caller_args_2<SelfClass, R, A0, A1, Member>::s_caller;

  /*--------------------------------------------------------------------------*/
  /** Cette classe dfinit un driv de explicit_method_caller pour appeler une
      mthode donne d'une classe donne. Cette dernire est dfinit par
      SelfClasse. L'adresse de la mthode est Member et doit prendre trois
      paramtres dont le type est donn dans ArgsList. */
  template< typename SelfClass, typename R, typename A0, typename A1,
            typename A2,
            R (SelfClass::*Member)(A0, A1, A2) >
  class method_caller_args_3
  {
  public:
    typedef R (SelfClass::*mem_fun_type)(A0, A1, A2);
    typedef typename SelfClass::script_context_type script_context_type;

  public:
    class caller_type:
      public explicit_method_caller<SelfClass>
    {
    private:
      void explicit_execute
      ( SelfClass& self, const std::vector<std::string>& args,
        const script_context_type& context ) const
      {
        const mem_fun_type member(Member);
        (self.*member)
          ( string_to_arg<script_context_type, A0>::convert(context, args[0]),
            string_to_arg<script_context_type, A1>::convert(context, args[1]),
            string_to_arg<script_context_type, A2>::convert(context, args[2]) );
      }
    }; // class caller_type;

  public:
    /** Une seule instance de caller_type est suffisante dans la mesure o il
        n'y a pas de variables membres (s_caller ne sert qu' faire la
        transition vers la mthode relle). On pourra ainsi utiliser son adresse
        et viter des allocations dynamiques. */
    static const caller_type s_caller;

  }; // class method_caller_args_3

  /*--------------------------------------------------------------------------*/
  /** Dfinition de method_caller_args_3::s_caller pour les paramtres de
      patron de classes donns. */
  template< typename SelfClass, typename R, typename A0, typename A1,
            typename A2,
            R (SelfClass::*Member)(A0, A1, A2) >
  const typename
  method_caller_args_3<SelfClass, R, A0, A1, A2, Member>::caller_type
  method_caller_args_3<SelfClass, R, A0, A1, A2, Member>::s_caller;

/** Si on souhaite appeler des mthodes avec quatre paramtres, il suffira de
    dfinir un nouveau couple de classes method_caller_args_4. De mme pour plus
    de quatre paramtres.
*/
} // namespace script

#endif // __SCRIPT_METHOD_CALLER_ARGS_HPP__
